@@ -4,18 +4,20 @@ use warnings;
 
 package MooseX::Enumeration::Meta::Method::Accessor::Native::Enumeration::is;
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.003';
+our $VERSION   = '0.004';
 
 use Moose::Role;
 with 'Moose::Meta::Method::Accessor::Native::Reader';
 
-sub _minimum_arguments { 1 }
-sub _maximum_arguments { 1 }
+around _minimum_arguments => sub { 1 };
+around _maximum_arguments => sub { 1 };
 
 sub _return_value
 {
 	my $self = shift;
 	my ($slot_access) = @_;
+	# Note that $_[0] comes from @curried which has been closed over
+	# and contains the string we need to compare against.
 	return $slot_access . ' eq $_[0]';
 }
 
@@ -26,18 +28,26 @@ around _generate_method => sub
 	
 	my @curried = @{ $self->curried_arguments };
 	
-	if (@curried==1 and defined $curried[0] and not ref $curried[0])
+	# If everything is as expected...
+	if ( @curried==1
+	and defined $curried[0]
+	and not ref $curried[0]
+	and $self->_maximum_arguments==1
+	and $self->_minimum_arguments==1 )
 	{
+		# ... then provide a highly optimized accessor.
 		require B;
 		require Moose::Util;
 		return sprintf(
-			'sub { @_ > 1 and %s; %s eq %s }',
+			'sub { %s if @_ > 1; %s eq %s }',
 			"Moose::Util::throw_exception('MethodExpectsFewerArgs', 'method_name', 'is', 'maximum_args', 1)",
 			$self->_get_value('$_[0]'),
 			B::perlstring($curried[0]),
 		);
 	}
 	
+	# Otherwise we should trust the default implementation
+	# from Moose::Meta::Method::Accessor::Native::Reader.
 	$self->$next(@_);
 };
 
